"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.epicCurrencyList = void 0;
const epicCurrency_1 = require("./epicCurrency");
class epicCurrencyList {
    // Constructor
    constructor(currencyList, standardCurrencyCode) {
        this.currencyList = {};
        this.changeStandard = (currencyCode) => {
            if (this.standard.val().code.toLowerCase() == currencyCode.toLowerCase())
                return this;
            let currency;
            if (currency = this.currencyList[currencyCode.toLowerCase()]) {
                let standard = this.standard;
                this.standard = currency;
                this.addCurrency(standard);
                this.removeCurrency(currencyCode.toLowerCase());
                for (let c in this.currencyList)
                    if (this.currencyList.hasOwnProperty(c))
                        this.currencyList[c].changeRate(this.currencyList[c].val().rate / currency.val().rate);
                this.standard.changeRate(1);
            }
            else
                throw new Error("Invalid Currency Code. Currency Not Found!");
            return this;
        };
        this.addCurrency = (currency) => {
            if (currency instanceof epicCurrency_1.epicCurrency) {
                if (!this.standard)
                    this.currencyList[currency.val().code.toLowerCase()] = currency;
                else if (this.standard.val().code.toLowerCase() !== currency.val().code.toLowerCase())
                    this.currencyList[currency.val().code.toLowerCase()] = currency;
                else
                    this.standard = currency.changeRate(1);
            }
            else if (currency &&
                typeof currency === "object" &&
                currency["code"] &&
                currency["name"] &&
                currency["rate"] &&
                currency["symbol"])
                this.addCurrency(new epicCurrency_1.epicCurrency(currency.code, currency.name, currency.rate, currency.symbol, currency.image));
            else
                throw new Error("Invalid Currency Object Has Been Received!");
            return this;
        };
        this.addCurrencies = (currencies) => {
            if (currencies instanceof Array)
                currencies.forEach((v) => this.addCurrency(v));
            else if (currencies && typeof currencies === "object")
                Object.values(currencies).forEach((v) => this.addCurrency(v));
            else
                throw new Error("Invalid Currencies Provided!");
            return this;
        };
        this.removeCurrency = (currencyCode) => {
            delete this.currencyList[currencyCode.toLowerCase()];
            return this;
        };
        this.removeCurrencies = (currencyCodes) => {
            if (currencyCodes instanceof Array)
                currencyCodes.forEach(v => this.removeCurrency(v));
            else
                throw new Error("Invalid Currency Codes Array Provided!");
            return this;
        };
        this.getList = (includeStandard = true, clone = false) => {
            let currencyList = this.currencyList;
            if (includeStandard)
                currencyList[this.standard.val().code.toLowerCase()] = this.standard;
            if (clone) {
                // Create A Cloned Currency List
                const newList = {};
                for (let i in currencyList)
                    if (currencyList.hasOwnProperty(i))
                        newList[i] = currencyList[i].clone();
                // Re-Assign Cloned Data
                currencyList = newList;
            }
            return currencyList;
        };
        this.getListJson = (includeStandard = true, stringify = false) => {
            const list = {};
            Object.values(this.getList(includeStandard)).forEach(currency => {
                list[currency.val().code] = currency.val();
            });
            if (stringify)
                return JSON.stringify(list);
            else
                return list;
        };
        this.getStandard = () => this.standard;
        this.get = (currencyCode, includeStandard = true) => {
            if (!this.has(currencyCode, includeStandard))
                throw new Error("Invalid Currency Code! Currency Not Found.");
            return this.getList(includeStandard)[currencyCode.toLowerCase()];
        };
        this.has = (currencyCode, includeStandard = true) => {
            const currency = this.getList(includeStandard)[currencyCode.toLowerCase()];
            return (currency && currency instanceof epicCurrency_1.epicCurrency);
        };
        this.clone = () => new epicCurrencyList(this.getList(true, true), this.standard.val().code);
        if (currencyList)
            if (currencyList instanceof epicCurrencyList) {
                const resolvedCurrency = currencyList.changeStandard(standardCurrencyCode);
                this.standard = resolvedCurrency.getStandard();
                this.currencyList = resolvedCurrency.getList(false);
            }
            else if (currencyList && (currencyList instanceof Array || typeof currencyList === "object"))
                this.addCurrencies(currencyList);
            else
                throw new Error("Invalid Currency List Type Provided!");
        const gotStandard = this.get(standardCurrencyCode, false);
        if (gotStandard.val().rate !== 1)
            throw new Error("Standard Currency Should Always Have Rate Of 1!");
        else {
            // Set Initial Standard
            this.removeCurrency(standardCurrencyCode);
            this.standard = gotStandard;
        }
        return this;
    }
}
exports.epicCurrencyList = epicCurrencyList;
//# sourceMappingURL=epicCurrencyList.js.map