"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.epicMoneyStack = void 0;
const epicMoney_1 = require("./epicMoney");
const epicCurrency_1 = require("./epicCurrency");
const epicCurrencyList_1 = require("./epicCurrencyList");
class epicMoneyStack {
    // Constructor
    constructor(currencyList) {
        this.moneyList = [];
        this.add = (money, options = { description: "N/A", quantity: 1 }) => {
            if (!(money instanceof epicMoney_1.epicMoney))
                throw new Error("Invalid Money Object Provided!");
            // Create New Instance Of Money
            money = money.clone();
            // Add To The Stack
            this.moneyList.push({
                description: options.description,
                quantity: parseInt(options.quantity),
                type: "ADD",
                money: money,
                total: money.multiply(parseInt(options.quantity)),
                history: this.total(),
            });
            return this;
        };
        this.deduct = (money, options = { description: "N/A", quantity: 1 }) => {
            if (!(money instanceof epicMoney_1.epicMoney))
                throw new Error("Invalid Money Object Provided!");
            // Create New Instance Of Money
            money = money.clone();
            // Add To The Stack
            this.moneyList.push({
                description: options.description,
                quantity: parseInt(options.quantity),
                type: "DEDUCT",
                money: money.invert(),
                total: money.multiply(parseInt(options.quantity)),
                history: this.total(),
            });
            return this;
        };
        this.stacks = () => this.moneyList;
        this.export = () => this.stacks().map(stack => ({
            description: stack.description,
            currency: stack.money.getCurrency().val().code,
            quantity: stack.quantity,
            amount: stack.money.val(),
            total: stack.total.val(),
        }));
        this.import = (source) => {
            source.forEach(row => {
                this.add(new epicMoney_1.epicMoney(row.amount || 0, this.currencyList.get(row.currency || this.currencyList.getStandard().val().code)), {
                    description: row.description || "N/A",
                    quantity: parseInt((row.quantity || 1)),
                });
            });
            return this;
        };
        this.total = (currency) => {
            if (currency) {
                if (!(currency instanceof epicCurrency_1.epicCurrency))
                    throw new Error("Invalid Currency Object Provided!");
            }
            else
                currency = this.currencyList.getStandard();
            const total = new epicMoney_1.epicMoney(0, currency);
            this.stacks().forEach(stack => {
                total.plus(stack.total);
            });
            return total;
        };
        if (currencyList && currencyList instanceof epicCurrencyList_1.epicCurrencyList)
            this.currencyList = currencyList;
        else
            throw new Error("A Valid Currency List Object Is Required!");
        return this;
    }
}
exports.epicMoneyStack = epicMoneyStack;
//# sourceMappingURL=epicMoneyStack.js.map